@@ -22,23 +22,17 @@
 
   CHANGE HISTORY:
 
-    20 December 2016 -- by Jean-Romain Roussel -- Change fprint(stderr, ...), raise an exeption
-
     20 March 2011 -- added capability for *.zip, *.rar, and *.7z on Windows
     12 December 2003 -- adapted from Stefan Gumhold's SIGGRAPH submission hack
 
 ===============================================================================
 */
 
+#define STRICT_R_HEADERS
+#include <R.h>
+
 #include <stdio.h>
-
-#if defined(__clang__)
-  #include <string>
-#else
-  #include <string.h>
-#endif
-
-#include <stdexcept>
+#include <string.h>
 
 #ifdef _WIN32
 #include <io.h>
@@ -61,7 +55,7 @@ static FILE* fopen7zipped(const char* filename, const char* mode)
 		int hPipe[2];
 		if (_pipe(hPipe, 2048, ((mode[1] =='b') ? _O_BINARY : _O_TEXT) | _O_NOINHERIT) == -1)
 		{
-			throw std::runtime_error(std::string("could not create pipe"));
+			REprintf( "could not create pipe\n");
 			return NULL;
 		}
 
@@ -71,14 +65,14 @@ static FILE* fopen7zipped(const char* filename, const char* mode)
 		// make the write end of pipe go to stdout
 		if (_dup2(hPipe[WRITE_HANDLE], _fileno(stdout)) != 0)
 		{
-			throw std::runtime_error(std::string("could not set pipe output"));
+			REprintf( "could not set pipe output\n");
 			return NULL;
 		}
 
 		// redirect read end of pipe to input file
 		if (_dup2(hPipe[READ_HANDLE], _fileno(stdin)) != 0)
 		{
-			throw std::runtime_error(std::string("could not redirect input file"));
+			REprintf( "could not redirect input file\n");
 			return NULL;
 		}
 
@@ -91,7 +85,7 @@ static FILE* fopen7zipped(const char* filename, const char* mode)
 		// redirect stdout back into stdout
 		if (_dup2(hStdOut, _fileno(stdout)) != 0)
 		{
-			throw std::runtime_error(std::string("could not reconstruct stdout"));
+			REprintf( "could not reconstruct stdout\n");
 			return NULL;
 		}
 
@@ -113,7 +107,7 @@ static FILE* fopenZIPped(const char* filename, const char* mode)
 		int hPipe[2];
 		if (_pipe(hPipe, 2048, ((mode[1] =='b') ? _O_BINARY : _O_TEXT) | _O_NOINHERIT) == -1)
 		{
-			throw std::runtime_error(std::string("could not create pipe"));
+			REprintf( "could not create pipe\n");
 			return NULL;
 		}
 
@@ -123,14 +117,14 @@ static FILE* fopenZIPped(const char* filename, const char* mode)
 		// make the write end of pipe go to stdout
 		if (_dup2(hPipe[WRITE_HANDLE], _fileno(stdout)) != 0)
 		{
-			throw std::runtime_error(std::string("could not set pipe output"));
+			REprintf( "could not set pipe output\n");
 			return NULL;
 		}
 
 		// redirect read end of pipe to input file
 		if (_dup2(hPipe[READ_HANDLE], _fileno(stdin)) != 0)
 		{
-			throw std::runtime_error(std::string("could not redirect input file"));
+			REprintf( "could not redirect input file\n");
 			return NULL;
 		}
 
@@ -143,7 +137,7 @@ static FILE* fopenZIPped(const char* filename, const char* mode)
 		// redirect stdout back into stdout
 		if (_dup2(hStdOut, _fileno(stdout)) != 0)
 		{
-			throw std::runtime_error(std::string("could not reconstruct stdout"));
+			REprintf( "could not reconstruct stdout\n");
 			return NULL;
 		}
 
@@ -169,7 +163,7 @@ static FILE* fopenGzipped(const char* filename, const char* mode)
 		int hPipe[2];
 		if (_pipe(hPipe, 2048, ((mode[1] =='b') ? _O_BINARY : _O_TEXT) | _O_NOINHERIT) == -1)
 		{
-			throw std::runtime_error(std::string("could not create pipe"));
+			REprintf( "could not create pipe\n");
 			return NULL;
 		}
 
@@ -178,7 +172,7 @@ static FILE* fopenGzipped(const char* filename, const char* mode)
 		// redirect stdin to input file
 		if (_dup2(_fileno(gzipInput), _fileno(stdin)) != 0)
 		{
-			throw std::runtime_error(std::string("could not redirect stdin"));
+			REprintf( "could not redirect stdin\n");
 			return NULL;
 		}
 
@@ -187,7 +181,7 @@ static FILE* fopenGzipped(const char* filename, const char* mode)
 		// redirect stdout to write end of pipe
 		if (_dup2(hPipe[WRITE_HANDLE], _fileno(stdout)) != 0)
 		{
-			throw std::runtime_error(std::string("could not set pipe output"));
+			REprintf( "could not set pipe output\n");
 			return NULL;
 		}
 
@@ -197,7 +191,7 @@ static FILE* fopenGzipped(const char* filename, const char* mode)
 		// redirect read end of pipe to input file
 		if (_dup2(hPipe[READ_HANDLE], _fileno(gzipInput)) != 0)
 		{
-			throw std::runtime_error(std::string("could not redirect input file"));
+			REprintf( "could not redirect input file\n");
 			return NULL;
 		}
 
@@ -210,14 +204,14 @@ static FILE* fopenGzipped(const char* filename, const char* mode)
 		// redirect stdin back into stdin
 		if (_dup2(hStdIn, _fileno(stdin)) != 0)
 		{
-			throw std::runtime_error(std::string("could not reconstruct stdin"));
+			REprintf( "could not reconstruct stdin\n");
 			return NULL;
 		}
 
 		// redirect stdout back into stdout
 		if (_dup2(hStdOut, _fileno(stdout)) != 0)
 		{
-			throw std::runtime_error(std::string("could not reconstruct stdout"));
+			REprintf( "could not reconstruct stdout\n");
 			return NULL;
 		}
 
@@ -239,7 +233,7 @@ static FILE* fopenGzippedNew(const char* filename, const char* mode)
 		int hPipe[2];
 		if (_pipe(hPipe, 2048, ((mode[1] =='b') ? _O_BINARY : _O_TEXT) | _O_NOINHERIT) == -1)
 		{
-			throw std::runtime_error(std::string("could not create pipe"));
+			REprintf( "could not create pipe\n");
 			return NULL;
 		}
 
@@ -249,14 +243,14 @@ static FILE* fopenGzippedNew(const char* filename, const char* mode)
 		// make the write end of pipe go to stdout
 		if (_dup2(hPipe[WRITE_HANDLE], _fileno(stdout)) != 0)
 		{
-			throw std::runtime_error(std::string("could not set pipe output"));
+			REprintf( "could not set pipe output\n");
 			return NULL;
 		}
 
 		// redirect read end of pipe to input file
 		if (_dup2(hPipe[READ_HANDLE], _fileno(stdin)) != 0)
 		{
-			throw std::runtime_error(std::string("could not redirect input file"));
+			REprintf( "could not redirect input file\n");
 			return NULL;
 		}
 
@@ -269,7 +263,7 @@ static FILE* fopenGzippedNew(const char* filename, const char* mode)
 		// redirect stdout back into stdout
 		if (_dup2(hStdOut, _fileno(stdout)) != 0)
 		{
-			throw std::runtime_error(std::string("could not reconstruct stdout"));
+			REprintf( "could not reconstruct stdout\n");
 			return NULL;
 		}
 
@@ -291,7 +285,7 @@ static FILE* fopenRARed(const char* filename, const char* mode)
 		int hPipe[2];
 		if (_pipe(hPipe, 2048, ((mode[1] =='b') ? _O_BINARY : _O_TEXT) | _O_NOINHERIT) == -1)
 		{
-			throw std::runtime_error(std::string("could not create pipe"));
+			REprintf( "could not create pipe\n");
 			return NULL;
 		}
 
@@ -301,14 +295,14 @@ static FILE* fopenRARed(const char* filename, const char* mode)
 		// make the write end of pipe go to stdout
 		if (_dup2(hPipe[WRITE_HANDLE], _fileno(stdout)) != 0)
 		{
-			throw std::runtime_error(std::string("could not set pipe output"));
+			REprintf( "could not set pipe output\n");
 			return NULL;
 		}
 
 		// redirect read end of pipe to input file
 		if (_dup2(hPipe[READ_HANDLE], _fileno(stdin)) != 0)
 		{
-			throw std::runtime_error(std::string("could not redirect input file"));
+			REprintf( "could not redirect input file\n");
 			return NULL;
 		}
 
@@ -321,7 +315,7 @@ static FILE* fopenRARed(const char* filename, const char* mode)
 		// redirect stdout back into stdout
 		if (_dup2(hStdOut, _fileno(stdout)) != 0)
 		{
-			throw std::runtime_error(std::string("could not reconstruct stdout"));
+			REprintf( "could not reconstruct stdout\n");
 			return NULL;
 		}
 
@@ -347,7 +341,7 @@ FILE* fopen_compressed(const char* filename, const char* mode, bool* piped)
     file = fopenGzipped(filename, mode);
     if (piped) *piped = true;
 #else
-    throw std::runtime_error(std::string("ERROR: no support for gzipped input"));
+    REprintf( "ERROR: no support for gzipped input\n");
     return 0;
 #endif
   }
@@ -357,7 +351,7 @@ FILE* fopen_compressed(const char* filename, const char* mode, bool* piped)
     file = fopenZIPped(filename, mode);
     if (piped) *piped = true;
 #else
-    throw std::runtime_error(std::string("ERROR: no support for ZIPped input"));
+    REprintf( "ERROR: no support for ZIPped input\n");
     return 0;
 #endif
   }
@@ -367,7 +361,7 @@ FILE* fopen_compressed(const char* filename, const char* mode, bool* piped)
     file = fopen7zipped(filename, mode);
     if (piped) *piped = true;
 #else
-    throw std::runtime_error(std::string("ERROR: no support for 7zipped input"));
+    REprintf( "ERROR: no support for 7zipped input\n");
     return 0;
 #endif
   }
@@ -377,7 +371,7 @@ FILE* fopen_compressed(const char* filename, const char* mode, bool* piped)
     file = fopenRARed(filename, mode);
     if (piped) *piped = true;
 #else
-    throw std::runtime_error(std::string("ERROR: no support for RARed input"));
+    REprintf( "ERROR: no support for RARed input\n");
     return 0;
 #endif
   }
