@@ -26,6 +26,8 @@
 
     see corresponding header file
 
+    6  may  2018 -- by Jean-Romain Roussel - l1036-1040 use #prama to skip wrong -Wstringop-overflow
+
 ===============================================================================
 */
 #include "laswriter.hpp"
@@ -1034,7 +1036,8 @@ void LASwriteOpener::cut_characters(U32 cut)
     if ((len == 0) || (file_name[len] == '\\') || (file_name[len] == '/') || (file_name[len] == ':'))
     {
       len = strlen(file_name);
-      strncpy(new_file_name, file_name, len-cut);
+      size_t s = len-cut;
+      strncpy(new_file_name, file_name, s);
     }
     else
     {
