@@ -107,6 +107,109 @@ void RLASstreamer::setfilter(CharacterVector filter)
   return;
 }
 
+void RLASstreamer::select(CharacterVector string)
+{
+  std::string select = as<std::string>(string);
+  std::string unselect = as<std::string>(string);
+
+  t = false;
+  i = false;
+  r = false;
+  n = false;
+  d = false;
+  e = false;
+  c = false;
+  s = false;
+  k = false;
+  w = false;
+  a = false;
+  u = false;
+  p = false;
+  rgb = false;
+  nir = false;
+
+  if (select.find("*") != std::string::npos)
+    select = "xyztirndecskwaupRGBN0";
+
+  if (select.find("i") != std::string::npos) read_i(true);
+  if (select.find("t") != std::string::npos) read_t(true);
+  if (select.find("r") != std::string::npos) read_r(true);
+  if (select.find("n") != std::string::npos) read_n(true);
+  if (select.find("d") != std::string::npos) read_d(true);
+  if (select.find("e") != std::string::npos) read_e(true);
+  if (select.find("c") != std::string::npos) read_c(true);
+  if (select.find("s") != std::string::npos) read_s(true);
+  if (select.find("k") != std::string::npos) read_k(true);
+  if (select.find("w") != std::string::npos) read_w(true);
+  if (select.find("a") != std::string::npos) read_a(true);
+  if (select.find("u") != std::string::npos) read_u(true);
+  if (select.find("p") != std::string::npos) read_p(true);
+  if (select.find("R") != std::string::npos) read_rgb(true);
+  if (select.find("G") != std::string::npos) read_rgb(true);
+  if (select.find("B") != std::string::npos) read_rgb(true);
+  if (select.find("N") != std::string::npos) read_nir(true);
+
+  if (unselect.find("-i") != std::string::npos) read_i(false);
+  if (unselect.find("-t") != std::string::npos) read_t(false);
+  if (unselect.find("-r") != std::string::npos) read_r(false);
+  if (unselect.find("-n") != std::string::npos) read_n(false);
+  if (unselect.find("-d") != std::string::npos) read_d(false);
+  if (unselect.find("-e") != std::string::npos) read_e(false);
+  if (unselect.find("-c") != std::string::npos) read_c(false);
+  if (unselect.find("-s") != std::string::npos) read_s(false);
+  if (unselect.find("-k") != std::string::npos) read_k(false);
+  if (unselect.find("-w") != std::string::npos) read_w(false);
+  if (unselect.find("-a") != std::string::npos) read_a(false);
+  if (unselect.find("-u") != std::string::npos) read_u(false);
+  if (unselect.find("-p") != std::string::npos) read_p(false);
+  if (unselect.find("-R") != std::string::npos) read_rgb(false);
+  if (unselect.find("-G") != std::string::npos) read_rgb(false);
+  if (unselect.find("-B") != std::string::npos) read_rgb(false);
+  if (unselect.find("-N") != std::string::npos) read_nir(false);
+
+  std::vector<bool> select_eb(9);
+  std::fill(select_eb.begin(), select_eb.end(), false);
+
+  if (select.find("0") != std::string::npos)
+    std::fill(select_eb.begin(), select_eb.end(), true);
+  else
+  {
+    if (select.find("1") != std::string::npos) select_eb[0] = true;
+    if (select.find("2") != std::string::npos) select_eb[1] = true;
+    if (select.find("3") != std::string::npos) select_eb[2] = true;
+    if (select.find("4") != std::string::npos) select_eb[3] = true;
+    if (select.find("5") != std::string::npos) select_eb[4] = true;
+    if (select.find("6") != std::string::npos) select_eb[5] = true;
+    if (select.find("7") != std::string::npos) select_eb[6] = true;
+    if (select.find("8") != std::string::npos) select_eb[7] = true;
+    if (select.find("9") != std::string::npos) select_eb[8] = true;
+  }
+
+  if (unselect.find("-0") != std::string::npos)
+    std::fill(select_eb.begin(), select_eb.end(), false);
+  else
+  {
+    if (unselect.find("-1") != std::string::npos) select_eb[0] = false;
+    if (unselect.find("-2") != std::string::npos) select_eb[1] = false;
+    if (unselect.find("-3") != std::string::npos) select_eb[2] = false;
+    if (unselect.find("-4") != std::string::npos) select_eb[3] = false;
+    if (unselect.find("-5") != std::string::npos) select_eb[4] = false;
+    if (unselect.find("-6") != std::string::npos) select_eb[5] = false;
+    if (unselect.find("-7") != std::string::npos) select_eb[6] = false;
+    if (unselect.find("-8") != std::string::npos) select_eb[7] = false;
+    if (unselect.find("-9") != std::string::npos) select_eb[8] = false;
+  }
+
+  IntegerVector pos_eb;
+  for (int i = 0 ; i < select_eb.size() ; i++)
+  {
+    if(select_eb[i])
+      pos_eb.push_back(i);
+  }
+
+  read_eb(pos_eb);
+}
+
 void RLASstreamer::initialize()
 {
   // Intialize the reader
@@ -143,6 +246,8 @@ void RLASstreamer::initialize()
       nalloc = npoints;
   }
 
+  nsynthetic = 0;
+  nwithheld = 0;
   initialized = true;
 
   return;
@@ -164,6 +269,9 @@ void RLASstreamer::allocation()
     if(d) SDF.reserve(nalloc);
     if(e) EoF.reserve(nalloc);
     if(c) C.reserve(nalloc);
+    if(s) Synthetic.reserve(nalloc);
+    if(k) Keypoint.reserve(nalloc);
+    if(w) Withheld.reserve(nalloc);
     if(a) SA.reserve(nalloc);
     if(u) UD.reserve(nalloc);
     if(p) PSI.reserve(nalloc);
@@ -252,6 +360,9 @@ void RLASstreamer::write_point()
     if(d) SDF.push_back(lasreader->point.get_scan_direction_flag());
     if(e) EoF.push_back(lasreader->point.get_edge_of_flight_line());
     if(c) C.push_back(lasreader->point.get_classification());
+    if(s) Synthetic.push_back(lasreader->point.get_synthetic_flag());
+    if(k) Keypoint.push_back(lasreader->point.get_keypoint_flag());
+    if(w) Withheld.push_back(lasreader->point.get_withheld_flag());
     if(a) SA.push_back(lasreader->point.get_scan_angle_rank());
     if(u) UD.push_back(lasreader->point.get_user_data());
     if(p) PSI.push_back(lasreader->point.get_point_source_ID());
@@ -267,6 +378,12 @@ void RLASstreamer::write_point()
     {
       extra_bytes_attr[j].push_back(&lasreader->point);
     }
+
+    if (lasreader->point.get_synthetic_flag())
+      nsynthetic++;
+
+    if(lasreader->point.get_withheld_flag())
+      nwithheld++;
   }
 }
 
@@ -370,6 +487,30 @@ List RLASstreamer::terminate()
       C.shrink_to_fit();
     }
 
+    if(s)
+    {
+      lasdata.push_back(Synthetic);
+      field.push_back("Synthetic_flag");
+      Synthetic.clear();
+      Synthetic.shrink_to_fit();
+    }
+
+    if(k)
+    {
+      lasdata.push_back(Keypoint);
+      field.push_back("Keypoint_flag");
+      Keypoint.clear();
+      Keypoint.shrink_to_fit();
+    }
+
+    if(w)
+    {
+      lasdata.push_back(Withheld);
+      field.push_back("Withheld_flag");
+      Withheld.clear();
+      Withheld.shrink_to_fit();
+    }
+
     if(a)
     {
       lasdata.push_back(SA);
@@ -438,6 +579,12 @@ List RLASstreamer::terminate()
 
     lasdata.names() = field;
 
+    if (nwithheld > 0)
+      Rcerr << "Warning: there are " << nwithheld << " points flagged 'withheld'." << std::endl;
+
+    if (nsynthetic > 0)
+      Rcerr << "Warning: there are " << nsynthetic << " points flagged 'synthetic'."  << std::endl;
+
     return(lasdata);
   }
 }
@@ -451,6 +598,9 @@ void RLASstreamer::initialize_bool()
   d = true;
   e = true;
   c = true;
+  s = true;
+  k = true;
+  w = true;
   a = true;
   u = true;
   p = true;
@@ -472,6 +622,9 @@ void RLASstreamer::read_n(bool b){ n = b; }
 void RLASstreamer::read_d(bool b){ d = b; }
 void RLASstreamer::read_e(bool b){ e = b; }
 void RLASstreamer::read_c(bool b){ c = b; }
+void RLASstreamer::read_s(bool b){ s = b; }
+void RLASstreamer::read_k(bool b){ k = b; }
+void RLASstreamer::read_w(bool b){ w = b; }
 void RLASstreamer::read_a(bool b){ a = b; }
 void RLASstreamer::read_u(bool b){ u = b; }
 void RLASstreamer::read_p(bool b){ p = b; }
@@ -554,6 +707,8 @@ RLASExtrabyteAttributes::RLASExtrabyteAttributes()
   has_max = false;
   has_offset = false;
   has_no_data = false;
+  scale = 1.0;
+  offset = 0.0;
 }
 
 bool RLASExtrabyteAttributes::is_supported(){ return(type <= 10); }
